@@ -28,8 +28,6 @@
 #include "gtktreeprivate.h"
 #include "gtkalias.h"
 
-static void gtk_cell_renderer_text_init       (GtkCellRendererText      *celltext);
-static void gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class);
 static void gtk_cell_renderer_text_finalize   (GObject                  *object);
 
 static void gtk_cell_renderer_text_get_property  (GObject                  *object,
@@ -77,6 +75,7 @@ enum {
   PROP_SINGLE_PARAGRAPH_MODE,
   PROP_WIDTH_CHARS,
   PROP_WRAP_WIDTH,
+  PROP_ALIGN,
   
   /* Style args */
   PROP_BACKGROUND,
@@ -119,7 +118,6 @@ enum {
   PROP_ELLIPSIZE_SET
 };
 
-static gpointer parent_class;
 static guint text_cell_renderer_signals [LAST_SIGNAL];
 
 #define GTK_CELL_RENDERER_TEXT_PATH "gtk-cell-renderer-text-path"
@@ -133,11 +131,13 @@ struct _GtkCellRendererTextPrivate
   guint language_set : 1;
   guint markup_set : 1;
   guint ellipsize_set : 1;
+  guint align_set : 1;
   
   gulong focus_out_id;
   PangoLanguage *language;
   PangoEllipsizeMode ellipsize;
   PangoWrapMode wrap_mode;
+  PangoAlignment align;
   
   gulong populate_popup_id;
   gulong entry_menu_popdown_timeout;
@@ -149,34 +149,7 @@ struct _GtkCellRendererTextPrivate
   GtkWidget *entry;
 };
 
-
-GType
-gtk_cell_renderer_text_get_type (void)
-{
-  static GType cell_text_type = 0;
-
-  if (!cell_text_type)
-    {
-      static const GTypeInfo cell_text_info =
-      {
-        sizeof (GtkCellRendererTextClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_cell_renderer_text_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkCellRendererText),
-	0,              /* n_preallocs */
-        (GInstanceInitFunc) gtk_cell_renderer_text_init,
-      };
-
-      cell_text_type =
-	g_type_register_static (GTK_TYPE_CELL_RENDERER, "GtkCellRendererText",
-				&cell_text_info, 0);
-    }
-
-  return cell_text_type;
-}
+G_DEFINE_TYPE (GtkCellRendererText, gtk_cell_renderer_text, GTK_TYPE_CELL_RENDERER)
 
 static void
 gtk_cell_renderer_text_init (GtkCellRendererText *celltext)
@@ -194,6 +167,8 @@ gtk_cell_renderer_text_init (GtkCellRendererText *celltext)
 
   priv->width_chars = -1;
   priv->wrap_width = -1;
+  priv->align = PANGO_ALIGN_LEFT;
+  priv->align_set = FALSE;
 }
 
 static void
@@ -202,8 +177,6 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS (class);
 
-  parent_class = g_type_class_peek_parent (class);
-  
   object_class->finalize = gtk_cell_renderer_text_finalize;
   
   object_class->get_property = gtk_cell_renderer_text_get_property;
@@ -291,7 +264,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                    PROP_FONT,
                                    g_param_spec_string ("font",
                                                         P_("Font"),
-                                                        P_("Font description as a string"),
+                                                        P_("Font description as a string, e.g. \"Sans Italic 12\""),
                                                         NULL,
                                                         GTK_PARAM_READWRITE));
 
@@ -435,7 +408,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
 						      P_("Ellipsize"),
 						      P_("The preferred place to ellipsize the string, "
 							 "if the cell renderer does not have enough room "
-							 "to display the entire string, if at all"),
+							 "to display the entire string"),
 						      PANGO_TYPE_ELLIPSIZE_MODE,
 						      PANGO_ELLIPSIZE_NONE,
 						      GTK_PARAM_READWRITE));
@@ -498,6 +471,21 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
 						     -1,
 						     GTK_PARAM_READWRITE));
 
+  /**
+   * GtkCellRendererText:alignment:
+   *
+   * Specifies how to align the lines of text.
+   *
+   * Since: 2.10
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_ALIGN,
+                                   g_param_spec_enum ("alignment",
+						      P_("Alignment"),
+						      P_("How to align the lines"),
+						      PANGO_TYPE_ALIGNMENT,
+						      PANGO_ALIGN_LEFT,
+						      GTK_PARAM_READWRITE));
   
   /* Style props are set or not */
 
@@ -564,7 +552,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                 P_("Whether this tag affects the ellipsize mode"));
 
   text_cell_renderer_signals [EDITED] =
-    g_signal_new ("edited",
+    g_signal_new (I_("edited"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkCellRendererTextClass, edited),
@@ -596,7 +584,7 @@ gtk_cell_renderer_text_finalize (GObject *object)
   if (priv->language)
     g_object_unref (priv->language);
 
-  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  (* G_OBJECT_CLASS (gtk_cell_renderer_text_parent_class)->finalize) (object);
 }
 
 static PangoFontMask
@@ -671,14 +659,7 @@ gtk_cell_renderer_text_get_property (GObject        *object,
       break;
 
     case PROP_FONT:
-      {
-        /* FIXME GValue imposes a totally gratuitous string copy
-         * here, we could just hand off string ownership
-         */
-        gchar *str = pango_font_description_to_string (celltext->font);
-        g_value_set_string (value, str);
-        g_free (str);
-      }
+        g_value_take_string (value, pango_font_description_to_string (celltext->font));
       break;
       
     case PROP_FONT_DESC:
@@ -749,6 +730,10 @@ gtk_cell_renderer_text_get_property (GObject        *object,
       g_value_set_int (value, priv->wrap_width);
       break;
       
+    case PROP_ALIGN:
+      g_value_set_enum (value, priv->align);
+      break;
+
     case PROP_BACKGROUND_SET:
       g_value_set_boolean (value, celltext->background_set);
       break;
@@ -1042,7 +1027,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
 					NULL,
 					&error))
 	  {
-	    g_warning ("Failed to set cell text from markup due to error parsing markup: %s",
+	    g_warning ("Failed to set text from markup due to error parsing markup: %s",
 		       error->message);
 	    g_error_free (error);
 	    return;
@@ -1249,6 +1234,11 @@ gtk_cell_renderer_text_set_property (GObject      *object,
       priv->width_chars = g_value_get_int (value);
       break;  
 
+    case PROP_ALIGN:
+      priv->align = g_value_get_enum (value);
+      priv->align_set = TRUE;
+      break;
+
     case PROP_BACKGROUND_SET:
       celltext->background_set = g_value_get_boolean (value);
       break;
@@ -1446,6 +1436,20 @@ get_layout (GtkCellRendererText *celltext,
       pango_layout_set_wrap (layout, PANGO_WRAP_CHAR);
     }
 
+  if (priv->align_set)
+    pango_layout_set_alignment (layout, priv->align);
+  else
+    {
+      PangoAlignment align;
+
+      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+	align = PANGO_ALIGN_RIGHT;
+      else
+	align = PANGO_ALIGN_LEFT;
+
+      pango_layout_set_alignment (layout, align);
+    }
+
   pango_layout_set_attributes (layout, attr_list);
 
   pango_attr_list_unref (attr_list);
@@ -1476,8 +1480,8 @@ get_size (GtkCellRenderer *cell,
       PangoFontDescription *font_desc;
       gint row_height;
 
-      font_desc = pango_font_description_copy (widget->style->font_desc);
-      pango_font_description_merge (font_desc, celltext->font, TRUE);
+      font_desc = pango_font_description_copy_static (widget->style->font_desc);
+      pango_font_description_merge_static (font_desc, celltext->font, TRUE);
 
       if (celltext->scale_set)
 	pango_font_description_set_size (font_desc,
@@ -1550,7 +1554,7 @@ get_size (GtkCellRenderer *cell,
 	  else 
 	    *x_offset = cell->xalign * (cell_area->width - (rect.x + rect.width + (2 * cell->xpad)));
 
-	  if (priv->ellipsize_set || priv->wrap_width != -1)
+	  if ((priv->ellipsize_set && priv->ellipsize != PANGO_ELLIPSIZE_NONE) || priv->wrap_width != -1)
 	    *x_offset = MAX(*x_offset, 0);
 	}
       if (y_offset)
@@ -1644,7 +1648,7 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
       cairo_destroy (cr);
     }
 
-  if (priv->ellipsize_set)
+  if (priv->ellipsize_set && priv->ellipsize != PANGO_ELLIPSIZE_NONE)
     pango_layout_set_width (layout, 
 			    (cell_area->width - x_offset - 2 * cell->xpad) * PANGO_SCALE);
   else if (priv->wrap_width == -1)
@@ -1789,6 +1793,7 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
 				      GdkRectangle         *cell_area,
 				      GtkCellRendererState  flags)
 {
+  GtkRequisition requisition;
   GtkCellRendererText *celltext;
   GtkCellRendererTextPrivate *priv;
 
@@ -1806,10 +1811,36 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
 
   if (celltext->text)
     gtk_entry_set_text (GTK_ENTRY (priv->entry), celltext->text);
-  g_object_set_data_full (G_OBJECT (priv->entry), GTK_CELL_RENDERER_TEXT_PATH, g_strdup (path), g_free);
+  g_object_set_data_full (G_OBJECT (priv->entry), I_(GTK_CELL_RENDERER_TEXT_PATH), g_strdup (path), g_free);
   
   gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
   
+  gtk_widget_size_request (priv->entry, &requisition);
+  if (requisition.height < cell_area->height)
+    {
+      GtkBorder *style_border;
+      GtkBorder border;
+
+      gtk_widget_style_get (priv->entry,
+			    "inner-border", &style_border,
+			    NULL);
+
+      if (style_border)
+        {
+	  border = *style_border;
+	  g_boxed_free (GTK_TYPE_BORDER, style_border);
+	}
+      else
+        {
+	  /* Since boxed style properties can't have default values ... */
+	  border.left = 2;
+	  border.right = 2;
+	}
+
+      border.top = (cell_area->height - requisition.height) / 2;
+      border.bottom = (cell_area->height - requisition.height) / 2;
+      gtk_entry_set_inner_border (GTK_ENTRY (priv->entry), &border);
+    }
 
   priv->in_entry_menu = FALSE;
   if (priv->entry_menu_popdown_timeout)
@@ -1822,9 +1853,9 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
 		    "editing_done",
 		    G_CALLBACK (gtk_cell_renderer_text_editing_done),
 		    celltext);
-  priv->focus_out_id = g_signal_connect (priv->entry, "focus_out_event",
-		         G_CALLBACK (gtk_cell_renderer_text_focus_out_event),
-		         celltext);
+  priv->focus_out_id = g_signal_connect_after (priv->entry, "focus_out_event",
+					       G_CALLBACK (gtk_cell_renderer_text_focus_out_event),
+					       celltext);
   priv->populate_popup_id =
     g_signal_connect (priv->entry, "populate_popup",
                       G_CALLBACK (gtk_cell_renderer_text_populate_popup),
