@@ -81,9 +81,9 @@ _gtk_file_chooser_embed_set_delegate (GtkFileChooserEmbed *receiver,
   
   g_object_set_data (G_OBJECT (receiver), I_("gtk-file-chooser-embed-delegate"), delegate);
 
-  g_signal_connect (delegate, "default_size_changed",
+  g_signal_connect (delegate, "default-size-changed",
 		    G_CALLBACK (delegate_default_size_changed), receiver);
-  g_signal_connect (delegate, "response_requested",
+  g_signal_connect (delegate, "response-requested",
 		    G_CALLBACK (delegate_response_requested), receiver);
 }
 
@@ -171,7 +171,7 @@ gtk_file_chooser_embed_class_init (gpointer g_iface)
 		NULL, NULL,
 		_gtk_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
-  g_signal_new (_("response-requested"),
+  g_signal_new (I_("response-requested"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserEmbedIface, response_requested),
