@@ -136,6 +136,7 @@ struct GtkPrintUnixDialogPrivate
   GtkWidget *print_at_radio;
   GtkWidget *print_at_entry;
   GtkWidget *print_hold_radio;
+  GtkWidget *preview_button;
   gboolean updating_print_at;
   GtkPrinterOptionWidget *pages_per_sheet;
   GtkPrinterOptionWidget *duplex;
@@ -273,8 +274,13 @@ gtk_print_unix_dialog_init (GtkPrintUnixDialog *dialog)
 		    (GCallback) gtk_print_unix_dialog_destroy, 
 		    NULL);
 
+  priv->preview_button = gtk_button_new_from_stock (GTK_STOCK_PRINT_PREVIEW);
+  gtk_widget_show (priv->preview_button);
+   
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), 
+                                priv->preview_button, 
+                                GTK_RESPONSE_APPLY);
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-			  GTK_STOCK_PRINT_PREVIEW, GTK_RESPONSE_APPLY, 
 			  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			  GTK_STOCK_PRINT, GTK_RESPONSE_OK,
                           NULL);
@@ -1029,6 +1035,11 @@ update_dialog_from_capabilities (GtkPrintUnixDialog *dialog)
   gtk_widget_set_sensitive (priv->scale_spin,
 			    caps & GTK_PRINT_CAPABILITY_PAGE_SET);
 
+  if (caps & GTK_PRINT_CAPABILITY_PREVIEW)
+    gtk_widget_show (priv->preview_button);
+  else
+    gtk_widget_hide (priv->preview_button);
+
   update_collate_icon (NULL, dialog);
 
   gtk_tree_model_filter_refilter (priv->printer_list_filter);
@@ -2581,7 +2592,10 @@ gtk_print_unix_dialog_set_settings (GtkPrintUnixDialog *dialog,
       dialog_set_print_pages (dialog, gtk_print_settings_get_print_pages (settings));
       ranges = gtk_print_settings_get_page_ranges (settings, &num_ranges);
       if (ranges)
-	dialog_set_page_ranges (dialog, ranges, num_ranges);
+        {
+	  dialog_set_page_ranges (dialog, ranges, num_ranges);
+          g_free (ranges);
+        }
 
       priv->format_for_printer =
 	g_strdup (gtk_print_settings_get (settings, "format-for-printer"));
@@ -2680,6 +2694,16 @@ gtk_print_unix_dialog_get_settings (GtkPrintUnixDialog *dialog)
   return settings;
 }
 
+/**
+ * gtk_print_unix_dialog_add_custom_tab:
+ * @dialog: a #GtkPrintUnixDialog
+ * @child: the widget to put in the custom tab
+ * @tab_label: the widget to use as tab label
+ *
+ * Adds a custom tab to the print dialog.
+ *
+ * Since: 2.10
+ */
 void
 gtk_print_unix_dialog_add_custom_tab (GtkPrintUnixDialog *dialog,
 				      GtkWidget          *child,
@@ -2694,7 +2718,7 @@ gtk_print_unix_dialog_add_custom_tab (GtkPrintUnixDialog *dialog,
 /**
  * gtk_print_unix_dialog_set_manual_capabilities:
  * @dialog: a #GtkPrintUnixDialog
- * @capabilites: the printing capabilities of your application
+ * @capabilities: the printing capabilities of your application
  *
  * This lets you specify the printing capabilities your application
  * supports. For instance, if you can handle scaling the output then
