@@ -142,7 +142,7 @@ G_DEFINE_TYPE_WITH_CODE (GtkTreeStore, gtk_tree_store, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_DEST,
 						gtk_tree_store_drag_dest_init)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE,
-						gtk_tree_store_sortable_init));
+						gtk_tree_store_sortable_init))
 
 static void
 gtk_tree_store_class_init (GtkTreeStoreClass *class)
@@ -248,8 +248,7 @@ gtk_tree_store_new (gint n_columns,
       GType type = va_arg (args, GType);
       if (! _gtk_tree_data_list_check_type (type))
 	{
-	  g_warning ("%s: Invalid type %s passed to gtk_tree_store_new_with_types\n",
-		     G_STRLOC, g_type_name (type));
+	  g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (type));
 	  g_object_unref (retval);
 	  return NULL;
 	}
@@ -284,8 +283,7 @@ gtk_tree_store_newv (gint   n_columns,
     {
       if (! _gtk_tree_data_list_check_type (types[i]))
 	{
-	  g_warning ("%s: Invalid type %s passed to gtk_tree_store_new_with_types\n",
-		     G_STRLOC, g_type_name (types[i]));
+	  g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (types[i]));
 	  g_object_unref (retval);
 	  return NULL;
 	}
@@ -322,7 +320,7 @@ gtk_tree_store_set_column_types (GtkTreeStore *tree_store,
     {
       if (! _gtk_tree_data_list_check_type (types[i]))
 	{
-	  g_warning ("%s: Invalid type %s passed to gtk_tree_store_set_column_types\n", G_STRLOC, g_type_name (types[i]));
+	  g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (types[i]));
 	  continue;
 	}
       gtk_tree_store_set_column_type (tree_store, i, types[i]);
@@ -378,7 +376,7 @@ gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
 {
   if (!_gtk_tree_data_list_check_type (type))
     {
-      g_warning ("%s: Invalid type %s passed to gtk_tree_store_new_with_types\n", G_STRLOC, g_type_name (type));
+      g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (type));
       return;
     }
   tree_store->column_headers[column] = type;
@@ -840,6 +838,31 @@ gtk_tree_store_set_value (GtkTreeStore *tree_store,
     }
 }
 
+static GtkTreeIterCompareFunc
+gtk_tree_store_get_compare_func (GtkTreeStore *tree_store)
+{
+  GtkTreeIterCompareFunc func = NULL;
+
+  if (GTK_TREE_STORE_IS_SORTED (tree_store))
+    {
+      if (tree_store->sort_column_id != -1)
+	{
+	  GtkTreeDataSortHeader *header;
+	  header = _gtk_tree_data_list_get_header (tree_store->sort_list,
+						   tree_store->sort_column_id);
+	  g_return_val_if_fail (header != NULL, NULL);
+	  g_return_val_if_fail (header->func != NULL, NULL);
+	  func = header->func;
+	}
+      else
+	{
+	  func = tree_store->default_sort_func;
+	}
+    }
+
+  return func;
+}
+
 static void
 gtk_tree_store_set_valist_internal (GtkTreeStore *tree_store,
                                     GtkTreeIter  *iter,
@@ -852,23 +875,7 @@ gtk_tree_store_set_valist_internal (GtkTreeStore *tree_store,
 
   column = va_arg (var_args, gint);
 
-  if (GTK_TREE_STORE_IS_SORTED (tree_store))
-    {
-      if (tree_store->sort_column_id != -1)
-	{
-	  GtkTreeDataSortHeader *header;
-	  header = _gtk_tree_data_list_get_header (tree_store->sort_list,
-						   tree_store->sort_column_id);
-	  g_return_if_fail (header != NULL);
-	  g_return_if_fail (header->func != NULL);
-	  func = header->func;
-	}
-      else
-	{
-	  func = tree_store->default_sort_func;
-	}
-    }
-
+  func = gtk_tree_store_get_compare_func (tree_store);
   if (func != _gtk_tree_data_list_compare_func)
     *maybe_need_sort = TRUE;
 
@@ -1268,7 +1275,7 @@ gtk_tree_store_insert_after (GtkTreeStore *tree_store,
 /**
  * gtk_tree_store_insert_with_values:
  * @tree_store: A #GtkTreeStore
- * @iter: An unset #GtkTreeIter to set the new row
+ * @iter: An unset #GtkTreeIter to set the new row, or %NULL.
  * @parent: A valid #GtkTreeIter, or %NULL
  * @position: position to insert the new row
  * @Varargs: pairs of column number and value, terminated with -1
@@ -1304,12 +1311,16 @@ gtk_tree_store_insert_with_values (GtkTreeStore *tree_store,
   GtkTreePath *path;
   GNode *parent_node;
   GNode *new_node;
+  GtkTreeIter tmp_iter;
   va_list var_args;
   gboolean changed = FALSE;
   gboolean maybe_need_sort = FALSE;
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-  g_return_if_fail (iter != NULL);
+
+  if (!iter)
+    iter = &tmp_iter;
+
   if (parent)
     g_return_if_fail (VALID_ITER (parent, tree_store));
 
@@ -1355,7 +1366,7 @@ gtk_tree_store_insert_with_values (GtkTreeStore *tree_store,
 /**
  * gtk_tree_store_insert_with_valuesv:
  * @tree_store: A #GtkTreeStore
- * @iter: An unset #GtkTreeIter to set the new row
+ * @iter: An unset #GtkTreeIter to set the new row, or %NULL.
  * @parent: A valid #GtkTreeIter, or %NULL
  * @position: position to insert the new row
  * @columns: an array of column numbers
@@ -1380,13 +1391,17 @@ gtk_tree_store_insert_with_valuesv (GtkTreeStore *tree_store,
   GtkTreePath *path;
   GNode *parent_node;
   GNode *new_node;
+  GtkTreeIter tmp_iter;
   gboolean changed = FALSE;
   gboolean maybe_need_sort = FALSE;
   GtkTreeIterCompareFunc func = NULL;
   gint i;
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-  g_return_if_fail (iter != NULL);
+
+  if (!iter)
+    iter = &tmp_iter;
+
   if (parent)
     g_return_if_fail (VALID_ITER (parent, tree_store));
 
@@ -1403,23 +1418,7 @@ gtk_tree_store_insert_with_valuesv (GtkTreeStore *tree_store,
   iter->user_data = new_node;
   g_node_insert (parent_node, position, new_node);
 
-  if (GTK_TREE_STORE_IS_SORTED (tree_store))
-    {
-      if (tree_store->sort_column_id != -1)
-        {
-	  GtkTreeDataSortHeader *header;
-	  header = _gtk_tree_data_list_get_header (tree_store->sort_list,
-						   tree_store->sort_column_id);
-	  g_return_if_fail (header != NULL);
-	  g_return_if_fail (header->func != NULL);
-	  func = header->func;
-	}
-      else
-        {
-	  func = tree_store->default_sort_func;
-        }
-    }
-
+  func = gtk_tree_store_get_compare_func (tree_store);
   if (func != _gtk_tree_data_list_compare_func)
     maybe_need_sort = TRUE;
 
