@@ -1,7 +1,28 @@
+/* testtreeflow.c
+ * Copyright (C) 2001 Red Hat, Inc
+ * Author: Jonathan Blandford
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+#include <config.h>
 #include <gtk/gtk.h>
 
 GtkTreeModel *model = NULL;
-GRand *rand = NULL;
+static GRand *grand = NULL;
 GtkTreeSelection *selection = NULL;
 enum
 {
@@ -31,12 +52,12 @@ initialize_model (void)
   GtkTreeIter iter;
 
   model = (GtkTreeModel *) gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING);
-  rand = g_rand_new ();
+  grand = g_rand_new ();
   for (i = 0; i < NUM_ROWS; i++)
     {
       gtk_list_store_append (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			  TEXT_COLUMN, words[g_rand_int_range (rand, 0, NUM_WORDS)],
+			  TEXT_COLUMN, words[g_rand_int_range (grand, 0, NUM_WORDS)],
 			  -1);
     }
 }
@@ -49,7 +70,7 @@ futz_row (void)
   GtkTreeIter iter;
   GtkTreeIter iter2;
 
-  i = g_rand_int_range (rand, 0,
+  i = g_rand_int_range (grand, 0,
 			gtk_tree_model_iter_n_children (model, NULL));
   path = gtk_tree_path_new ();
   gtk_tree_path_append_index (path, i);
@@ -58,14 +79,14 @@ futz_row (void)
 
   if (gtk_tree_selection_iter_is_selected (selection, &iter))
     return;
-  switch (g_rand_int_range (rand, 0, 3))
+  switch (g_rand_int_range (grand, 0, 3))
     {
     case 0:
       /* insert */
             gtk_list_store_insert_after (GTK_LIST_STORE (model),
             				   &iter2, &iter);
             gtk_list_store_set (GTK_LIST_STORE (model), &iter2,
-            			  TEXT_COLUMN, words[g_rand_int_range (rand, 0, NUM_WORDS)],
+            			  TEXT_COLUMN, words[g_rand_int_range (grand, 0, NUM_WORDS)],
             			  -1);
       break;
     case 1:
@@ -80,7 +101,7 @@ futz_row (void)
       if (gtk_tree_model_iter_n_children (model, NULL) == 0)
 	return;
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-      			  TEXT_COLUMN, words[g_rand_int_range (rand, 0, NUM_WORDS)],
+      			  TEXT_COLUMN, words[g_rand_int_range (grand, 0, NUM_WORDS)],
       			  -1);
       break;
     }
